@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2020 Leonid "Bepis" Pavel
-# Copyright 2023 Mike Fährmann
+# Copyright 2023-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,78 +9,103 @@
 
 """Extractors for https://imgchest.com/"""
 
-from .common import GalleryExtractor
-from .. import text, exception
+from .common import GalleryExtractor, Extractor, Message
+from .. import text, util, exception
+
+BASE_PATTERN = r"(?:https?://)?(?:www\.)?imgchest\.com"
 
 
 class ImagechestGalleryExtractor(GalleryExtractor):
     """Extractor for image galleries from imgchest.com"""
     category = "imagechest"
     root = "https://imgchest.com"
-    pattern = r"(?:https?://)?(?:www\.)?imgchest\.com/p/([A-Za-z0-9]{11})"
+    pattern = BASE_PATTERN + r"/p/([A-Za-z0-9]{11})"
     example = "https://imgchest.com/p/abcdefghijk"
 
     def __init__(self, match):
-        self.gallery_id = match.group(1)
+        self.gallery_id = match[1]
         url = self.root + "/p/" + self.gallery_id
         GalleryExtractor.__init__(self, match, url)
 
     def _init(self):
-        access_token = self.config("access-token")
-        if access_token:
+        if access_token := self.config("access-token"):
             self.api = ImagechestAPI(self, access_token)
-            self.gallery_url = None
+            self.page_url = None
             self.metadata = self._metadata_api
-            self.images = self._images_api
 
     def metadata(self, page):
-        if "Sorry, but the page you requested could not be found." in page:
-            raise exception.NotFoundError("gallery")
+        try:
+            data = util.json_loads(text.unescape(text.extr(
+                page, 'data-page="', '"')))
+            post = data["props"]["post"]
+        except Exception:
+            if "<title>Not Found</title>" in page:
+                raise exception.NotFoundError("gallery")
+            self.files = ()
+            return {}
 
-        return {
-            "gallery_id": self.gallery_id,
-            "title": text.unescape(text.extr(
-                page, 'property="og:title" content="', '"').strip())
-        }
+        self.files = post.pop("files", ())
+        post["gallery_id"] = self.gallery_id
+        post["tags"] = [tag["name"] for tag in post["tags"]]
 
-    def images(self, page):
-        if " More Files</button>" in page:
-            url = "{}/p/{}/loadAll".format(self.root, self.gallery_id)
-            headers = {
-                "X-Requested-With": "XMLHttpRequest",
-                "Origin"          : self.root,
-                "Referer"         : self.gallery_url,
-            }
-            csrf_token = text.extr(page, 'name="csrf-token" content="', '"')
-            data = {"_token": csrf_token}
-            page += self.request(
-                url, method="POST", headers=headers, data=data).text
-
-        return [
-            (url, None)
-            for url in text.extract_iter(page, 'data-url="', '"')
-        ]
+        return post
 
     def _metadata_api(self, page):
         post = self.api.post(self.gallery_id)
 
-        post["date"] = text.parse_datetime(
-            post["created"], "%Y-%m-%dT%H:%M:%S.%fZ")
+        post["date"] = self.parse_datetime_iso(post["created"])
         for img in post["images"]:
-            img["date"] = text.parse_datetime(
-                img["created"], "%Y-%m-%dT%H:%M:%S.%fZ")
+            img["date"] = self.parse_datetime_iso(img["created"])
 
         post["gallery_id"] = self.gallery_id
         post.pop("image_count", None)
-        self._image_list = post.pop("images")
+        self.files = post.pop("images")
 
         return post
 
-    def _images_api(self, page):
-        return [
-            (img["link"], img)
-            for img in self._image_list
-        ]
+    def images(self, page):
+        try:
+            return [
+                (file["link"], file)
+                for file in self.files
+            ]
+        except Exception:
+            return ()
+
+
+class ImagechestUserExtractor(Extractor):
+    """Extractor for imgchest.com user profiles"""
+    category = "imagechest"
+    subcategory = "user"
+    root = "https://imgchest.com"
+    pattern = BASE_PATTERN + r"/u/([^/?#]+)"
+    example = "https://imgchest.com/u/USER"
+
+    def items(self):
+        url = self.root + "/api/posts"
+        params = {
+            "page"    : 1,
+            "sort"    : "new",
+            "tag"     : "",
+            "q"       : "",
+            "username": text.unquote(self.groups[0]),
+            "nsfw"    : "true",
+        }
+
+        while True:
+            try:
+                data = self.request_json(url, params=params)["data"]
+            except (TypeError, KeyError):
+                return
+
+            if not data:
+                return
+
+            for gallery in data:
+                gallery["_extractor"] = ImagechestGalleryExtractor
+                yield Message.Queue, gallery["link"], gallery
+
+            params["page"] += 1
 
 
 class ImagechestAPI():
@@ -124,4 +149,4 @@ class ImagechestAPI():
 
             else:
                 self.extractor.log.debug(response.text)
-                raise exception.StopExtraction("API request failed")
+                raise exception.AbortExtraction("API request failed")
