@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2025 Mike Fährmann
+# Copyright 2019-2026 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -21,7 +21,7 @@ from datetime import datetime
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
 from gallery_dl import extractor, output, path, util, exception  # noqa E402
-from gallery_dl import postprocessor, config, archive  # noqa E402
+from gallery_dl import postprocessor, config, archive, dt  # noqa E402
 from gallery_dl.postprocessor.common import PostProcessor  # noqa E402
 
 
@@ -78,6 +78,7 @@ class BasePostprocessorTest(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         cls.dir = tempfile.TemporaryDirectory()
+        config.clear()
         config.set((), "base-directory", cls.dir.name)
         cls.job = FakeJob()
 
@@ -195,7 +196,9 @@ class ExecTest(BasePostprocessorTest):
 
     def test_command_string(self):
         self._create({
-            "command": "echo {} {_path} {_directory} {_filename} && rm {};",
+            "command": "echo {} {_path} {_path_unc} {_temppath} "
+                       "{_directory} {_directory_unc} {_filename} "
+                       "&& rm {};",
         })
 
         with patch("gallery_dl.util.Popen") as p:
@@ -206,11 +209,14 @@ class ExecTest(BasePostprocessorTest):
 
         p.assert_called_once_with(
             (f"echo "
+             f"{self.pathfmt.path} "
+             f"{self.pathfmt.path} "
              f"{self.pathfmt.realpath} "
-             f"{self.pathfmt.realpath} "
+             f"{self.pathfmt.temppath} "
+             f"{self.pathfmt.directory} "
              f"{self.pathfmt.realdirectory} "
              f"{self.pathfmt.filename} "
-             f"&& rm {self.pathfmt.realpath};"),
+             f"&& rm {self.pathfmt.path};"),
             shell=True,
             creationflags=0,
             start_new_session=False,
@@ -243,7 +249,8 @@ class ExecTest(BasePostprocessorTest):
     def test_command_many(self):
         self._create({
             "commands": [
-                "echo {} {_path} {_directory} {_filename} && rm {};",
+                "echo {} {_path} {_temppath} {_directory} {_filename} "
+                "&& rm {};",
                 ["~/script.sh", "{category}", "\fE _directory.upper()"],
             ]
         })
@@ -259,6 +266,7 @@ class ExecTest(BasePostprocessorTest):
                 (f"echo "
                  f"{self.pathfmt.realpath} "
                  f"{self.pathfmt.realpath} "
+                 f"{self.pathfmt.temppath} "
                  f"{self.pathfmt.realdirectory} "
                  f"{self.pathfmt.filename} "
                  f"&& rm {self.pathfmt.realpath};"),
@@ -370,6 +378,40 @@ class ExecTest(BasePostprocessorTest):
         m_aa.assert_called_once_with(self.pathfmt.kwdict)
         m_ac.assert_called_once()
 
+    def test_verbose_string(self):
+        self._create({
+            "command": "echo foo bar",
+            "verbose": False,
+        })
+
+        with patch("gallery_dl.util.Popen") as p, \
+                self.assertLogs(level=10) as log_info:
+            i = Mock()
+            i.wait.return_value = 123
+            p.return_value = i
+            self._trigger(("after",))
+
+        msg = "DEBUG:postprocessor.exec:Running 'echo'"
+        self.assertEqual(log_info.output[0], msg)
+        self.assertIn("'echo' returned with non-zero ", log_info.output[1])
+
+    def test_verbose_list(self):
+        self._create({
+            "command": ["echo", "foo", "bar"],
+            "verbose": False,
+        })
+
+        with patch("gallery_dl.util.Popen") as p, \
+                self.assertLogs(level=10) as log_info:
+            i = Mock()
+            i.wait.return_value = 123
+            p.return_value = i
+            self._trigger(("after",))
+
+        msg = "DEBUG:postprocessor.exec:Running 'echo'"
+        self.assertEqual(log_info.output[0], msg)
+        self.assertIn("'echo' returned with non-zero ", log_info.output[1])
+
 
 class HashTest(BasePostprocessorTest):
 
@@ -449,7 +491,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realpath}.JSON"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
         self.assertEqual(self._output(m), """{
     "category": "test",
@@ -469,6 +511,7 @@ class MetadataTest(BasePostprocessorTest):
             "indent"    : None,
             "open"      : "a",
             "encoding"  : "UTF-8",
+            "newline"   : "\r\n",
             "extension" : "JSON",
         }, {
             "public"    : "hello ワールド",
@@ -483,7 +526,9 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realpath}.JSON"
-        m.assert_called_once_with(path, "a", encoding="UTF-8")
+        m.assert_called_once_with(path, "a", encoding="UTF-8", newline='\r\n')
+        # Since we mocked the call to open,
+        # we don't actually see the effect of setting newline.
         self.assertEqual(self._output(m), """{\
 "_private" : "foo \\u30d0\\u30fc",\
 "category" : "test",\
@@ -504,7 +549,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realpath}.txt"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
         self.assertEqual(self._output(m), "foo\nbar\nbaz\n")
 
     def test_metadata_tags_split_1(self):
@@ -595,7 +640,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realdirectory}file.json"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_extfmt_2(self):
         self._create({
@@ -607,7 +652,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realdirectory}file.2.EXT-data:tESt"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_directory(self):
         self._create({
@@ -618,7 +663,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realdirectory}metadata/file.ext.json"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_directory_2(self):
         self._create({
@@ -630,7 +675,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realdirectory}metadata/file.json"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_directory_format(self):
         self._create(
@@ -642,7 +687,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realdirectory}../json/12500/file.ext.json"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_directory_empty(self):
         self._create(
@@ -653,7 +698,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realdirectory}./file.ext.json"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_basedirectory(self):
         self._create({"base-directory": True})
@@ -662,7 +707,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.basedirectory}file.ext.json"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_basedirectory_custom(self):
         self._create({
@@ -674,7 +719,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = "/home/test/meta/file.ext.json"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_filename(self):
         self._create({
@@ -686,7 +731,7 @@ class MetadataTest(BasePostprocessorTest):
             self._trigger()
 
         path = f"{self.pathfmt.realdirectory}test_file__meta_.data"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_meta_path(self):
         self._create({
@@ -786,7 +831,7 @@ class MetadataTest(BasePostprocessorTest):
         self.assertGreater(len(self._output(m)), 0)
 
         path = f"{self.pathfmt.realdirectory}file.ext.json"
-        m.assert_called_once_with(path, "w", encoding="utf-8")
+        m.assert_called_once_with(path, "w", encoding="utf-8", newline=None)
 
     def test_metadata_option_skip_false(self):
         self._create({"skip": False})
@@ -798,6 +843,28 @@ class MetadataTest(BasePostprocessorTest):
         self.assertTrue(not e.called)
         self.assertTrue(m.called)
 
+    def test_metadata_option_newline(self):
+        self._create({
+            "newline": "\r\n",
+            "filename"      : "data.json",
+            "directory"     : "",
+            "base-directory": self.dir.name,
+        })
+
+        self._trigger()
+
+        path = os.path.join(self.dir.name, "data.json")
+        with open(path, newline="") as fp:
+            content = fp.read()
+
+        self.assertEqual(content, """\
+{\r\n\
+    "category": "test",\r\n\
+    "filename": "file",\r\n\
+    "extension": "ext"\r\n\
+}\r\n\
+""")
+
     def test_metadata_option_include(self):
         self._create(
             {"include": ["_private", "filename", "foo"], "sort": True},
@@ -867,12 +934,22 @@ class MtimeTest(BasePostprocessorTest):
     def test_mtime_none(self):
         self._create(None, {"date": None})
         self._trigger()
-        self.assertNotIn("_mtime_meta", self.pathfmt.kwdict)
+        self.assertFalse(self.pathfmt.kwdict["_mtime_meta"])
+
+    def test_mtime_none_dt(self):
+        self._create(None, {"date": dt.NONE})
+        self._trigger()
+        self.assertFalse(self.pathfmt.kwdict["_mtime_meta"])
 
     def test_mtime_undefined(self):
         self._create(None, {})
         self._trigger()
-        self.assertNotIn("_mtime_meta", self.pathfmt.kwdict)
+        self.assertFalse(self.pathfmt.kwdict["_mtime_meta"])
+
+    def test_mtime_invalid(self):
+        self._create(None, {"date": "foobar"})
+        self._trigger()
+        self.assertFalse(self.pathfmt.kwdict["_mtime_meta"])
 
     def test_mtime_key(self):
         self._create({"key": "foo"}, {"foo": 315532800})
